#include <libmp/all.h>
#include <stdio.h>

#include "../cmd.h"
#include "../log.h"

bool cmd_list(lm_ctx_t *ctx, config_t *config, args_t *args){
  ssize_t count = 0;
  lm_pkg_t pkg;

  if(args_get_bool(args, "help")){
    info(_("Listing options for the install command:"));
    printf(_("    "FG_BOLD"--grep"FG_RESET":\tmakes the output \"grepable\"\n"));
    printf(_("    "FG_BOLD"--desc"FG_RESET":\tshow package descriptions\n"));
    return true;
  }
  
  while(lm_ctx_database_next(ctx, &pkg))
    count++;
  lm_ctx_database_next_free(ctx, &pkg);

  if(count <= 0 && !args_get_bool(args, "grep")){
    info(_("There no installed packages"));
    return true;
  }

  if(!args_get_bool(args, "grep"))
    info(count > 1 ? _("Listing %d packages") : _("Listing %d package"));

  while(lm_ctx_database_next(ctx, &pkg)){
    if(args_get_bool(args, "grep")){
      if(args_get_bool(args, "desc"))
        printf("%s:%s:%s\n", pkg.name, pkg.version, pkg.desc);
      else
        printf("%s:%s\n", pkg.name, pkg.version);
    }

    if(args_get_bool(args, "desc"))
      printf(FG_BOLD"%s ("FG_BLUE"%s"FG_RESET"): %s\n", pkg.name, pkg.version, pkg.desc);
    else
      printf(FG_BOLD"%s ("FG_BLUE"%s"FG_RESET")\n", pkg.name, pkg.version);
  }

  lm_ctx_database_next_free(ctx, &pkg);
  return true;
}
